@@ -13,8 +13,7 @@ import uuid
 
 import tornado.web
 from bson import ObjectId
-from utils import awaitable
-from utils import Dict
+from utils import Dict, awaitable
 
 from .application import Blueprint
 from .basehandler import BaseHandler
