@@ -14,7 +14,7 @@ from pathlib import Path
 
 import yaml
 
-from .utils import Dict
+from .base_utils import Dict
 
 
 class Config(Dict):
