@@ -7,11 +7,9 @@ Last modified: 2020-06-14 19:33:33
 import binascii
 
 import rsa
-from Crypto.Cipher import AES
-from Crypto.Cipher import DES
+from Crypto.Cipher import AES, DES
 
-from .utils import to_bytes
-from .utils import to_str
+from .base_utils import to_bytes, to_str
 
 
 class Crypt:
