@@ -10,7 +10,7 @@ import inspect
 import sys
 
 from .config_utils import Config
-from .utils import pprint
+from .base_utils import pprint
 
 
 def Fire(component=None):
@@ -39,6 +39,4 @@ def Fire(component=None):
     if inspect.isawaitable(ret):
         loop = asyncio.get_event_loop()
         ret = loop.run_until_complete(ret)
-
-    if ret is not None:
-        pprint(ret)
+    pprint(ret)
