@@ -6,6 +6,7 @@ Email: zhangkai@cmcm.com
 Last modified: 2018-01-05 11:27:36
 '''
 import logging
+from functools import lru_cache
 
 from tornado.log import LogFormatter
 
@@ -18,6 +19,7 @@ class WatchedFileHandler(logging.handlers.WatchedFileHandler):
             super(WatchedFileHandler, self).emit(record)
 
 
+@lru_cache()
 def Logger(filename=None, name=None, level='INFO', stream=True):
     logger = logging.getLogger(name)
     if logger.handlers and logger.handlers[0].name == 'Logger':
